package com.dky.calculate;


import com.dky.utils.entity.SysDeviceHeatScene;
import com.dky.utils.result.MatchedDevice;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

public class Scheme {

    public static List<Map<String, List<List<MatchedDevice>>>> calScheme(Double buildArea, List<SysDeviceHeatScene> list) {
        // 区分电锅炉、热泵
        Map<String, List<SysDeviceHeatScene>> grouped = list.stream()
                .collect(Collectors.groupingBy(SysDeviceHeatScene::getDevSubType));
        List<Map<String, List<List<MatchedDevice>>>> maps = new ArrayList<>();
        // 区分技术类型
        grouped.forEach((k, v) -> {
            Map<String, List<List<MatchedDevice>>> map = new HashMap<>();
            List<List<MatchedDevice>> planList = new ArrayList<>();
            Map<String, List<SysDeviceHeatScene>> collect = v.stream().collect(Collectors.groupingBy(SysDeviceHeatScene::getDevTechType));
            collect.forEach((k2, v2) -> {
                List<MatchedDevice> matchedDevices = calSchemeByTechType(buildArea, v2);
                planList.add(matchedDevices);
            });
            map.put(k, planList);
            maps.add(map);
        });
        return maps;
    }

    public static List<MatchedDevice> calSchemeByTechType(Double buildArea, List<SysDeviceHeatScene> list) {
        // 对List按照单台设备可参考供暖面积进行排序
        list.sort((o1, o2) -> Double.compare(o2.getDevReferenceArea(), o1.getDevReferenceArea()));
        Double remainArea = buildArea;
        // 遍历设备，根据建筑面积进行匹配设备数量
        List<MatchedDevice> matchedDeviceList = new ArrayList<>();
        for (SysDeviceHeatScene deviceHeatScene : list) {
            double v = remainArea / deviceHeatScene.getDevReferenceArea();
            int count = (int) (v);
            if (v < 1){
                count = count + 1;
            }
            remainArea = remainArea - count * deviceHeatScene.getDevReferenceArea();
            matchedDeviceList.add(new MatchedDevice(count, deviceHeatScene));
            if (remainArea <= 0){
                // 当总供暖面积已经满足需求时，跳出循环，不再选择其他设备
                break;
            }
        }
        return matchedDeviceList;
    }



}
